#include "stdafx.h"

__int64 PrimaryHashKeys[12][64],
		SecondaryHashKeys[12][64],
		EPHashKey,
		BlackMoveHashKey;
/*
Arrays of (unique) hash keys
required by transposition tables
 
PrimaryHashKeys[0][]  = for White King
PrimaryHashKeys[1][]  =           Queen
PrimaryHashKeys[2][]  =           Rook
PrimaryHashKeys[3][]  =           Bishop
PrimaryHashKeys[4][]  =			  Knight
PrimaryHashKeys[5][]  =		      Pawn

PrimaryHashKeys[6][]  = for Black King
PrimaryHashKeys[7][]  =			  Queen
PrimaryHashKeys[8][]  =			  Rook
PrimaryHashKeys[9][]  =			  Bishop
PrimaryHashKeys[10][] =			  Knight
PrimaryHashKeys[11][] =			  Pawn

SecondaryHashKeys[0][]  = for White King
SecondaryHashKeys[1][]  =			Queen
SecondaryHashKeys[2][]  =			Rook
SecondaryHashKeys[3][]  =			Bishop
SecondaryHashKeys[4][]  =			Knight
SecondaryHashKeys[5][]  =			Pawn

SecondaryHashKeys[6][]  = for Black King
SecondaryHashKeys[7][]  =			Queen
SecondaryHashKeys[8][]  =			Rook
SecondaryHashKeys[9][]  =			Bishop
SecondaryHashKeys[10][] =			Knight
SecondaryHashKeys[11][] =			Pawn
*/


///////////////////////////////////////////
//this function changes the moving side in the current position
///////////////////////////////////////////
void ChangeMovingSide(void)
{
	if (MovingSide == 0)
		MovingSide = 7;
	else 
		MovingSide = 0;
}


///////////////////////////////////////////
//this function prepares everything for the start
///////////////////////////////////////////
bool Initialize(void)
{
	int i=0,
		k=0,
		j=0;

	//Initializing BitBoards
	for (i=0;i<14;i++)
		BitBoards[i]=0;

	//Initialiazing Squares
	for (i=0;i<64;i++)
		Squares[i] = ( (__int64) 1) << i;

	//Initializing BitBoards for moves of all pieces from all squares
	for(k=0;k<64;k++)
	{
		for (i=0;i<8;i++)
		{
			//Knight's Moves
			if(KnightMoves[k][i] != -1)
			{
				KnightMovesBB[k] |= Squares[KnightMoves[k][i]];
			}

			//King's Moves
			if(KingMoves[k][i] != -1)
			{
				KingMovesBB[k] |= Squares[KingMoves[k][i]];
			}			
		}

		for (j=0;j<4;j++)
			for (i=0;i<7;i++)
			{
				//Rook's Moves
				if(RookMoves[k][j][i] != -1)
					RookMovesBB[k] |= Squares[RookMoves[k][j][i]];

				//Bishop's Moves
				if(BishopMoves[k][j][i] != -1)
					BishopMovesBB[k] |= Squares[BishopMoves[k][j][i]];
			}
	}

	//Initializing Move Lists
	for (i=0;i<MoveHistorySize;i++)
	{
		MoveList[i].fromSquare = -1;
		MoveList[i].toSquare = -1;
		MoveList[i].Type = -1;
		MoveList[i].AttackStatus = -100;
		MoveList[i].Score = -1000;
	}

	for (i = 0; i < 5; i++)
	{
		GoodMoves[i].fromSquare = -1;
		GoodMoves[i].toSquare = -1;
		GoodMoves[i].Type = -1;
		GoodMoves[i].AttackStatus = -100;
		GoodMoves[i].Score = -1000;

		BadMoves[i].fromSquare = -1;
		BadMoves[i].toSquare = -1;
		BadMoves[i].Type = -1;
		BadMoves[i].AttackStatus = -100;
		BadMoves[i].Score = 1000;
	}

	//Initializing Evaluation Parameters
	for (i=0;i<26;i++)
		Chromosome[i]=0;

	if (LoadParameters(Chromosome) == false)
		return false;

	
	//Initializing HashKeys
	int r1=0,
		r2=0;
	
	srand( (unsigned)time( NULL ) );
	
	for (i=0;i<12;i++)
		for(int j=0;j<64;j++)
		{
			r1 = rand() ;
			r2 = rand() ;
			PrimaryHashKeys[i][j] = (r1 << 16) + r2 ;
			
			r1 = rand() ;
			r2 = rand() ;
			SecondaryHashKeys[i][j] = (r1 << 16) + r2 ;
		}

	r1 = rand() ;
	r2 = rand() ;
	EPHashKey = (r1 << 16) + r2;

	r1 = rand() ;
	r2 = rand() ;
	BlackMoveHashKey = (r1 << 16) + r2;


	//Initializing Transposition Table
	for (i=0;i<TT_Size;i++)
	{
		TransTable[i].PHK = -1;
		TransTable[i].SHK = -1;

		TransTable[i].Depth = -1;
		TransTable[i].Value = -AB_Bound-1;
	}
	
	return true;
}


///////////////////////////////////////////
//this function prints the bit board in the following manner:
//
//a1...h1
//.......
//a8...h8
//That's why Inverse...
///////////////////////////////////////////
void InverseBitRepresentation( __int64 x)
{
	__int64 copy = x;
	
	for(int j=0;j<64;j++)
	{
		if ((j%8) == 0)
			printf("\n");

		if ( (copy & 01) )
			printf("1 ");
		else 
			printf("0 ");
			
		copy >>= 1;
	}
	printf("\n");
}


///////////////////////////////////////////
//this function returns the number of a piece on the square
//or -1 if the square is empty.
///////////////////////////////////////////
int WhichPiece(int square)
{
	int i=1;

	//empty square
	if(    ((Squares[square] & BitBoards[0]) == 0)  
		&& ((Squares[square] & BitBoards[7]) == 0)) 
		return -1;
	//not empty
	else 
		//white
		if((Squares[square] & BitBoards[0]) != 0)
		{
			while((Squares[square] & BitBoards[i]) == 0)
				i++;
		}
		//black
		else 
			if((Squares[square] & BitBoards[7]) != 0)
			{
				i=8;
				while((Squares[square] & BitBoards[i]) == 0)
					i++;
			}
return i;
}


///////////////////////////////////////////
//this function returns the number of square, on which the king of 
//the desired side stays.
//Assumes that the king is present!!!
///////////////////////////////////////////
int WhereKing(int side)
{
	int i=0;
	
	if (side == 0)
	{
		for (i=0; i<64; i++)
		{
			if ((Squares[i] & BitBoards[1]) != 0)
				//if the king is found, STOP
				break;
		}
	}

	if (side == 7)
	{
		for (i=0; i<64; i++)
		{
			if ((Squares[i] & BitBoards[8]) != 0)
				//if the king is found, STOP
				break;
		}
	}

return i;
}


///////////////////////////////////////////
//this function shows the current chessboard in "numberous" style, 
//i.e. white king = 1, white queen = 2, empty square = -1
///////////////////////////////////////////
void ShowBoard(void)
{
	printf("\nChessBoard\n");
	for (int i=7;i>=0;i--)
	{
		printf("\n");
		for(int j=0;j<8;j++)
		{
		int k=WhichPiece(i*8+j);
			printf("%3i ",k);
		}
	}
	printf("\n");
}


///////////////////////////////////////////
//this function returns the number of attacks 
//on the desired square by the desired side
///////////////////////////////////////////
int IsAttacked(int square, int ByWhichSide)
{
	int attacks=0;
	int j=0;
	
	//////////by whites
	if (ByWhichSide == 0)
	{
		// does any knight attack square?
		while(KnightMoves[square][j] != -1 && j<8)
		{	
			if (WhichPiece(KnightMoves[square][j]) == 5)
				attacks++;
			j++;
		}

		int direction=0;
		j=0;
		// do any queens or/and rooks attack square?
		for(direction=0; direction<4;direction++)
		{
			j=0;
			while(RookMoves[square][direction][j] != -1 && j<7)
			{
				//if there is a rook or a queen on the way,
				//it surely attacks
				if(WhichPiece(RookMoves[square][direction][j]) == 2 || 
				   WhichPiece(RookMoves[square][direction][j]) == 3)
				{
					attacks++;
					j++;
					continue;
				}
				else
					//if the current square is empty,
					//we move to the next square in this direction
					if(WhichPiece(RookMoves[square][direction][j]) == -1)
					{
						j++;
						continue;
					}
					//if the current square of this direction
					//is occupied by any other piece,
					//stop inspecting this direction and
					//move to the next one
					else 
						break;
			}//end while-loop
		}//end direction-for loop

		j=0;
		// do any queens or/and bishops attack square?
		for(direction=0; direction<4;direction++)
		{
			j=0;
			while(BishopMoves[square][direction][j] != -1 && j<7)
			{
				//if there is a bishop or a queen on the way,
				//it surely attacks
				if(WhichPiece(BishopMoves[square][direction][j]) == 2 || 
				   WhichPiece(BishopMoves[square][direction][j]) == 4)
				{
					attacks++;
					j++;
					continue;
				}
				else
					//if the current square is empty,
					//we move to the next square in this direction
					if(WhichPiece(BishopMoves[square][direction][j]) == -1)
					{
						j++;
						continue;
					}
					//if the current square of this direction
					//is occupied by any other piece,
					//stop inspecting this direction and
					//move to the next one
					else 
						break;
			}//end while-loop
		}//end direction-for loop

		
		//do pawns attack square?

		//if square is on the right vertical border
		//then only one pawn can attack it
		if (square == 23 || square == 31 || square == 39 ||
			square == 47 || square == 55 || square == 63)
		{
			if (WhichPiece(square-9) == 6)
			{
				attacks++;
					
				//are there also any bishops or queens on this pawn's diagonal ?
				j = 0;

				while(BishopMoves[square-9][3][j] != -1)
				{
					if (WhichPiece(BishopMoves[square-9][3][j]) == -1)
					{
						j++;
						continue;
					}
					else
						if ((WhichPiece(BishopMoves[square-9][3][j]) == 4) || 
							(WhichPiece(BishopMoves[square-9][3][j]) == 2))
						{
							attacks++;
							j++;
							continue;
						}
						else
							break;
				}
			}
		}
		//same for the left vertical border
		else 
			if (square == 16 || square == 24 || square == 32 ||
				square == 40 || square == 48 || square == 56)
			{
				if (WhichPiece(square-7) == 6)
					{
						attacks++;
						
						//are there also any bishops or queens on this pawn's diagonal ?
						j = 0;

						while(BishopMoves[square-7][2][j] != -1)
						{
							if (WhichPiece(BishopMoves[square-7][2][j]) == -1)
							{
								j++;
								continue;
							}
							else
								if ((WhichPiece(BishopMoves[square-7][2][j]) == 4) || 
									(WhichPiece(BishopMoves[square-7][2][j]) == 2))
								{
									attacks++;
									j++;
									continue;
								}
								else
									break;
						}
					}
			}

			//if square is somewhere in the middle,
			//then two pawns can attack it
			else 
				if (square > 16 )
				{
					if (WhichPiece(square-9) == 6)
					{
						attacks++;
						
						//are there also any bishops or queens on this pawn's diagonal ?
						j = 0;

						while(BishopMoves[square-9][3][j] != -1)
						{
							if (WhichPiece(BishopMoves[square-9][3][j]) == -1)
							{
								j++;
								continue;
							}
							else
								if ((WhichPiece(BishopMoves[square-9][3][j]) == 4) || 
									(WhichPiece(BishopMoves[square-9][3][j]) == 2))
								{
									attacks++;
									j++;
									continue;
								}
								else
									break;
						}
					}
					
					if (WhichPiece(square-7) == 6)
					{
						attacks++;
						
						//are there also any bishops or queens on this pawn's diagonal ?
						j = 0;

						while(BishopMoves[square-7][2][j] != -1)
						{
							if (WhichPiece(BishopMoves[square-7][2][j]) == -1)
							{
								j++;
								continue;
							}
							else
								if ((WhichPiece(BishopMoves[square-7][2][j]) == 4) || 
									(WhichPiece(BishopMoves[square-7][2][j]) == 2))
								{
									attacks++;
									j++;
									continue;
								}
								else
									break;
						}
					}
				}
		
		//does king attack square?
		j=0;
		while(KingMoves[square][j] != -1 && j<8)
		{	
			if (WhichPiece(KingMoves[square][j]) == 1)
			{
				attacks++;
				break; //there is only one king
			}
			j++;
		}
	}//end of white-if

	////////// by blacks
	if (ByWhichSide == 7)
	{
		// does any knight attack square?
		while(KnightMoves[square][j] != -1 && j<8)
		{	
			if (WhichPiece(KnightMoves[square][j]) == 12)
				attacks++;
			j++;
		}

		int direction=0;
		j=0;
		// do any queens or/and rooks attack square?
		for(direction=0; direction<4;direction++)
		{
			j=0;			
			while(RookMoves[square][direction][j] != -1 && j<7)
			{
				//if there is a rook or a queen on the way,
				//it surely attacks
				if(WhichPiece(RookMoves[square][direction][j]) == 9 || 
				   WhichPiece(RookMoves[square][direction][j]) == 10)
				{
					attacks++;
					j++;
					continue;
				}
				else
					//if the current square is empty,
					//we move to the next square in this direction
					if(WhichPiece(RookMoves[square][direction][j]) == -1)
					{
						j++;
						continue;
					}
					//if the current square of this direction
					//is occupied by any other piece,
					//stop inspecting this direction and
					//move to the next one
					else 
						break;
			}//end while-loop
		}//end direction-for loop

		j=0;
		// do any queens or/and bishops attack square?
		for(direction=0; direction<4;direction++)
		{
			j=0;			
			while(BishopMoves[square][direction][j] != -1 && j<7)
			{
				//if there is a bishop or a queen on the way,
				//it surely attacks
				if(WhichPiece(BishopMoves[square][direction][j]) == 9 || 
				   WhichPiece(BishopMoves[square][direction][j]) == 11)
				{
					attacks++;
					j++;
					continue;
				}
				else
					//if the current square is empty,
					//we move to the next square in this direction
					if(WhichPiece(BishopMoves[square][direction][j]) == -1)
					{
						j++;
						continue;
					}
					//if the current square of this direction
					//is occupied by any other piece,
					//stop inspecting this direction and
					//move to the next one
					else 
						break;
			}//end while-loop
		}//end direction-for loop

		//do pawns attack square?
		//
		//if square is on the right (from black's point of view)
		//vertical border then only one pawn can attack it
		if (square == 40 || square == 32 || square == 24 ||
			square == 16 || square ==  8 || square ==  0)
		{
			if (WhichPiece(square+9) == 13)
			{
				attacks++;
						
				//are there also any bishops or queens on this pawn's diagonal ?
				j = 0;

				while(BishopMoves[square+9][1][j] != -1)
				{
					if (WhichPiece(BishopMoves[square+9][1][j]) == -1)
					{
						j++;
						continue;
					}
					else
						if ((WhichPiece(BishopMoves[square+9][1][j]) == 11) || 
							(WhichPiece(BishopMoves[square+9][1][j]) == 9))
						{
							attacks++;
							j++;
							continue;
						}
						else
							break;
				}
			}
		}
		//same for the left (from black's point of view) vertical border
		else 
			if (square == 47 || square == 39 || square == 31 ||
				square == 23 || square == 15 || square ==  7)
			{
				if (WhichPiece(square+7) == 13)
					{
						attacks++;
						
						//are there also any bishops or queens on this pawn's diagonal ?
						j = 0;

						while(BishopMoves[square+7][0][j] != -1)
						{
							if (WhichPiece(BishopMoves[square+7][0][j]) == -1)
							{
								j++;
								continue;
							}
							else
								if ((WhichPiece(BishopMoves[square+7][0][j]) == 11) || 
									(WhichPiece(BishopMoves[square+7][0][j]) == 9))
								{
									attacks++;
									j++;
									continue;
								}
								else
									break;
						}
					}
			}

			//if square is somewhere in the middle,
			//then two pawns can attack it
			else 
				if (square < 47 )
				{
					if (WhichPiece(square+9) == 13)
					{
						attacks++;
						
						//are there also any bishops or queens on this pawn's diagonal ?
						j = 0;

						while(BishopMoves[square+9][1][j] != -1)
						{
							if (WhichPiece(BishopMoves[square+9][1][j]) == -1)
							{
								j++;
								continue;
							}
							else
								if ((WhichPiece(BishopMoves[square+9][1][j]) == 11) || 
									(WhichPiece(BishopMoves[square+9][1][j]) == 9))
								{
									attacks++;
									j++;
									continue;
								}
								else
									break;
						}
					}
					
					if (WhichPiece(square+7) == 13)
					{
						attacks++;
						
						//are there also any bishops or queens on this pawn's diagonal ?
						j = 0;

						while(BishopMoves[square+7][0][j] != -1)
						{
							if (WhichPiece(BishopMoves[square+7][0][j]) == -1)
							{
								j++;
								continue;
							}
							else
								if ((WhichPiece(BishopMoves[square+7][0][j]) == 11) || 
									(WhichPiece(BishopMoves[square+7][0][j]) == 9))
								{
									attacks++;
									j++;
									continue;
								}
								else
									break;
						}
					}
				}

		//does king attack square?
		j=0;
		while(KingMoves[square][j] != -1 && j<8)
		{	
			if (WhichPiece(KingMoves[square][j]) == 8)
			{
				attacks++;
				break; //there is only one king
			}
			j++;
		}

	}//end of black-if

	return attacks;
}


///////////////////////////////////////////
//this function adds the desired piece onto the desired square
///////////////////////////////////////////
void AddPiece(int square, int piece)
{
	//Modifying this piece's BB
	BitBoards[piece] |= Squares[square]; 
	
	//Modifying all piece's BB
	if (piece < 7)
		BitBoards[0] |= Squares[square];
	else
		BitBoards[7] |= Squares[square];
}


///////////////////////////////////////////
//this function removes certain piece from the mentioned square
///////////////////////////////////////////
void RemovePiece(int square, int piece)
{
	int i=0;
	
	//check, if the mentioned piece exists on the square
	i=WhichPiece(square);
	
	if (i != piece)
	{
		printf("Remove operation failed:\nNo %d  (there is %d instead) on %d!!!\n",piece,i,square);
		
		//visualised pause for the analysis
		ShowBoard();
		getchar();
	}
	else	
	{
		//Modifying this piece's BB
		BitBoards[piece] ^= Squares[square];  
	
		//Modifying all piece's BB
		if (piece < 7)
			BitBoards[0] ^= Squares[square];
		else
			BitBoards[7] ^= Squares[square];
	}
}


///////////////////////////////////////////
//this function creates the initial chess board position
///////////////////////////////////////////
void InitialPosition()
{
	
	int i=0;

	//Creating empty board
	for (i=0;i<14;i++)
		BitBoards[i] = 0;
	
	//White pieces
	AddPiece(4,1); //king
	AddPiece(3,2); //queen
	AddPiece(0,3); //rooks
	AddPiece(7,3);
	AddPiece(2,4); //bishops
	AddPiece(5,4);
	AddPiece(1,5); //knights
	AddPiece(6,5);

	//Black pieces
	AddPiece(60,8); //king
	AddPiece(59,9); //queen
	AddPiece(56,10);//rooks
	AddPiece(63,10);
	AddPiece(58,11);//bishops
	AddPiece(61,11);
	AddPiece(57,12);//knights
	AddPiece(62,12);

	//Pawns
	for(i=0;i<8;i++)
	{
		AddPiece(8+i,6);   //white
		AddPiece(48+i,13); //black
	}

	//both castlings are possible for both sides
	//in the beginning of the game
	WhiteLongCastling  = true;
	WhiteShortCastling = true; 
	BlackLongCastling  = true;
	BlackShortCastling = true;

	//no castlings done yet
	WhiteCastlingDone=false;
	BlackCastlingDone=false;
	
	//white to move
	MovingSide=0;
	
	//no EnPassant possibilities right now
	EnPassant = 0;
}


///////////////////////////////////////////
//this function determines the type of the move
///////////////////////////////////////////
int SetMoveType(int fromSquare, int toSquare)
{
		//piece on the departure square
	int dep = WhichPiece(fromSquare),
		//piece on the destination square
		dest= WhichPiece(toSquare);
	
	//castling move is when king is moved 
	//to TWO squares of its INITIAL location
	if ((dep == 1 || dep == 8) && abs(fromSquare-toSquare) == 2)
		MoveType=2;
	else
		if (dep ==  6 && dest == -1 )
			// white promotion
			if (toSquare > 55) 
				//toQueen
				MoveType = 401;
			//white EnPassant
			else 
				if (abs(fromSquare - toSquare)%8 != 0)
					MoveType = 3;
				//white usual pawn move
				else 
					MoveType = 0;
		else 
		{
			if (dep == 13 && dest == -1 )
				//black promotion
				if (toSquare <  8)
					//toQueen
					MoveType = 401;
				//black EnPassant
				else 
					if (abs(fromSquare - toSquare)%8 != 0)
						MoveType = 3;
					//black usual pawn move
					else 
						MoveType = 0;
			else
				//capture promotion
				if ((dep ==  6 && dest != -1 && toSquare > 55) ||
					(dep == 13 && dest != -1 && toSquare <  8))
					//to Queen
					MoveType = 411;
				else
					//capture move
					if (dest != -1)
						MoveType = 1;
					//usual move
					else 
						MoveType = 0;
		}
	return MoveType;
}


///////////////////////////////////////////
//This function makes the move,
//Type must match to the real type of fromSquare-toSquare move.
//So using Type = SetMoveType(fromSquare,toSquare) prevents the possible error of MoveType mismatch
///////////////////////////////////////////
bool Move(int fromSquare, int toSquare, int Type)
{
		//piece on the departure square
	int dep = WhichPiece(fromSquare),
		//piece on the destination square
		dest= WhichPiece(toSquare);
	
			//finds all legal moves of dep on fromSquare
	__int64 Validity = ComputeMoves(dep,fromSquare);

	//can dep move to toSquare indeed?
	if ((Validity & Squares[toSquare]) == 0)
		return false;
	
	//moving from nowhere or/and to nowhere?
	if ((fromSquare == -1) || (toSquare == -1))
		return false;
	
	//there is no piece to move
	if (dep == -1)
	{
		//see what's wrong
		printf("Move failed:\n no piece to move from %d\n",fromSquare);
		ShowBoard();
		getchar();

		return false;
	}
	
	//OK, the move exist

	//Ordinary Move
	if (Type == 0) 
	{			   
		RemovePiece(fromSquare,dep);
		AddPiece(toSquare, dep);
		
		//if after making the move king is on check
		//then the move is invalid, 
		//make it back and return false
		if (dep < 7)
		{
			if (IsAttacked(WhereKing(0),7) > 0)
			{
				RemovePiece(toSquare,dep);
				AddPiece(fromSquare,dep);
				return false;
			}
		}
		else
		{
			if (IsAttacked(WhereKing(7),0) > 0)
			{
				RemovePiece(toSquare,dep);
				AddPiece(fromSquare,dep);
				return false;
			}
		}
		
		//renewing castling possibilities
		bool CastlingDone;

		if (MovingSide == 0)
			CastlingDone=WhiteCastlingDone;
		else
			CastlingDone=BlackCastlingDone;

		if(CastlingDone == false)
		{
			//if not castled king was moved,
			//the moving side lost its castling possibilities
			if(dep == 1 && fromSquare == 4 && (WhiteShortCastling == true || WhiteLongCastling == true))
			{
				WhiteShortCastling = false;
				WhiteLongCastling = false;
			}
			if(dep == 8 && fromSquare == 60 && (BlackShortCastling == true || BlackLongCastling == true))
			{
				BlackShortCastling = false;
				BlackLongCastling = false;
			}
		
			//if the rook is moved,
			//the corresponding castling possibility is lost
		
			//white's long
			if (dep == 3 && fromSquare == 0 && WhiteLongCastling == true)
				WhiteLongCastling = false;				
			//white's short
			if (dep == 3 && fromSquare == 7 && WhiteShortCastling == true)
				WhiteShortCastling = false;				
			//black's long
			if (dep == 10 && fromSquare == 56 && BlackLongCastling == true)
				BlackLongCastling = false;
			//black's short
			if (dep == 10 && fromSquare == 63 && BlackShortCastling == true)
				BlackShortCastling = false;
		}

		//changing EnPassant status
		if ((dep == 6 || dep == 13) && (abs(fromSquare-toSquare) == 16))
			EnPassant = (fromSquare%8) + 1;
		else
			EnPassant=0;

		//give the right to move to the opponent
		ChangeMovingSide();
		
		return true;
	}
	
	//Capture Move
	if (Type == 1)
		if ((dep<7 && dest>7) || (dep>7 && dest<7))
		{
			RemovePiece(toSquare, dest);
			RemovePiece(fromSquare, dep);
			AddPiece(toSquare,dep);

			//if after making the move king is on check
			//then the move is invalid, 
			//make it back and return false
			if (dep < 7)
			{
				if (IsAttacked(WhereKing(0),7) > 0)
				{
					RemovePiece(toSquare,dep);
					AddPiece(fromSquare,dep);
					AddPiece(toSquare,dest);
					return false;
				}
			}
			else
			{
				if (IsAttacked(WhereKing(7),0) > 0)
				{
					RemovePiece(toSquare,dep);
					AddPiece(fromSquare,dep);
					AddPiece(toSquare,dest);
					return false;
				}
			}
			
			EnPassant = 0;

			//renewing castling possibilities			
			bool CastlingDone;

			//if a rook, which can castle, was captured,
			//the corresponding possibility is lost
			if (MovingSide == 0)
				CastlingDone=BlackCastlingDone;
			else
				CastlingDone=WhiteCastlingDone;

			if(CastlingDone == false)
			{
				if ((dest == 3) && (toSquare == 0) && (WhiteLongCastling == true))
					WhiteLongCastling = false;

				if ((dest == 3) && (toSquare == 7) && (WhiteShortCastling == true))
					WhiteShortCastling = false;

				if ((dest == 10) && (toSquare == 56) && (BlackLongCastling == true))
					BlackLongCastling = false;

				if ((dest == 10) && (toSquare == 63) && (BlackShortCastling == true))
					BlackShortCastling = false;
			}

			//if not castled king was moved,
			//the moving side lost its castling possibility
			if(dep == 1 && fromSquare == 4 && (WhiteShortCastling == true || WhiteLongCastling == true))
			{
				WhiteShortCastling = false;
				WhiteLongCastling = false;
			}
			if(dep == 8 && fromSquare == 60 && (BlackShortCastling == true || BlackLongCastling == true))
			{
				BlackShortCastling = false;
				BlackLongCastling = false;
			}
		
			//if the rook is moved,
			//the corresponding castling possibility is lost
			if (MovingSide == 0)
				CastlingDone=WhiteCastlingDone;
			else
				CastlingDone=BlackCastlingDone;
	
			if(CastlingDone == false)
			{
				if (MovingSide == 0)
					CastlingDone=WhiteCastlingDone;
				else
					CastlingDone=BlackCastlingDone;
				
				//white's long
				if (dep == 3 && fromSquare == 0 && WhiteLongCastling == true)
					WhiteLongCastling = false;				
				//white's short
				if (dep == 3 && fromSquare == 7 && WhiteShortCastling == true)
					WhiteShortCastling = false;				
				//black's long
				if (dep == 10 && fromSquare == 56 && BlackLongCastling == true)
					BlackLongCastling = false;
				//black's short
				if ((dep == 10) && (fromSquare == 63) && (BlackShortCastling == true))
					BlackShortCastling = false;
			}
			
			ChangeMovingSide();						
				return true;
		}
	
	//Castling Move
	//here no check possibility is considered,
	//since it is done in generating castling move
	if (Type == 2)
	{
		//White Castling
		if (fromSquare == 4)
		{
			//short 
			if (toSquare > fromSquare)
			{
				RemovePiece(fromSquare, dep);
				RemovePiece(fromSquare+3,3);
				AddPiece(toSquare,dep);
				AddPiece(toSquare-1,3);
				
				WhiteLongCastling = false;
				WhiteShortCastling = false;
	 			WhiteCastlingDone=true;
				EnPassant = 0;
				ChangeMovingSide();
				return true;
			}
			//long
			else
			{
				RemovePiece(fromSquare, dep);
				RemovePiece(fromSquare-4,3);
				AddPiece(toSquare,dep);
				AddPiece(toSquare+1,3);
				
				WhiteLongCastling = false;
				WhiteShortCastling = false;
				WhiteCastlingDone=true;
				EnPassant = 0;
				ChangeMovingSide();		
				return true;
			}
		}

		//Black Castling
		if (fromSquare == 60)
		{
			//short 
			if (toSquare > fromSquare)
			{
				RemovePiece(fromSquare, dep);
				RemovePiece(fromSquare+3,10);
				AddPiece(toSquare,dep);
				AddPiece(toSquare-1,10);
				
				BlackLongCastling = false;
				BlackShortCastling = false;
				BlackCastlingDone=true;
				EnPassant = 0;
				ChangeMovingSide();
				return true;
			}
			//long
			else
			{
				RemovePiece(fromSquare, dep);
				RemovePiece(fromSquare-4,10);
				AddPiece(toSquare,dep);
				AddPiece(toSquare+1,10);
				
				BlackLongCastling = false;
				BlackShortCastling = false;
				BlackCastlingDone=true;
				EnPassant = 0;
				ChangeMovingSide();
				return true;
			}
		}
	}
		
	//En Passant Capture Move
	if (Type == 3)
	{
		RemovePiece(fromSquare,dep);
		AddPiece(toSquare,dep);
		
		if(MovingSide == 0)
			RemovePiece(toSquare-8,13);
		else
			RemovePiece(toSquare+8,6);

		//if after making the move king is on check
		//then the move is invalid, 
		//make it back and return false
		if (dep < 7)
		{
			if (IsAttacked(WhereKing(0),7) > 0)
			{
				RemovePiece(toSquare,dep);
				AddPiece(fromSquare,dep);
				AddPiece(toSquare-8,13);
				return false;
			}
		}
		else
		{
			if (IsAttacked(WhereKing(7),0) > 0)
			{
				RemovePiece(toSquare,dep);
				AddPiece(fromSquare,dep);
				AddPiece(toSquare+8,6);
				return false;
			}
		}
		
		EnPassant = 0;
		ChangeMovingSide();
		return true;
	}

/*
//Ordinary Promotion Move
-----------REMINDER------------
Ordianry    toQueen  Move = 401
		    toRook   Move = 402
		    toBishop Move = 403
		    toKnight Move = 404

WithCapture toQueen  Move = 411
			toRook   Move = 412
			toBishop Move = 413
			toKnight Move = 414

Only toQueen are implemented
-------------------------------
*/
	//ordinary promotion
	if (Type == 401)
	{
		//white's
		if (fromSquare > 47 && fromSquare < 56)
		{
			RemovePiece(fromSquare,dep);
			AddPiece(toSquare,2);

			//if after making the move king is on check
			//then the move is invalid, 
			//make it back and return false
			if (IsAttacked(WhereKing(0),7) > 0)
			{
				RemovePiece(toSquare,2);
				AddPiece(fromSquare,dep);
				return false;
			}
			
			EnPassant = 0;			
			ChangeMovingSide();
			return true;
		}
		//black's
		if (fromSquare > 7 && fromSquare < 16)
		{
			RemovePiece(fromSquare,dep);
			AddPiece(toSquare,9);

			//if after making the move king is on check
			//then the move is invalid, 
			//make it back and return false
			if (IsAttacked(WhereKing(7),0) > 0)
			{
				RemovePiece(toSquare,9);
				AddPiece(fromSquare,dep);
				return false;
			}
			
			EnPassant = 0;			
			ChangeMovingSide();
			return true;
		}
	}

	//WithCapture promotion
	if (Type == 411)
	{
		//white's 
		if (fromSquare > 47 && fromSquare < 56)
		{
			RemovePiece(fromSquare,dep);
			RemovePiece(toSquare,dest);
			AddPiece(toSquare,2);

			//if after making the move king is on check
			//then the move is invalid, 
			//make it back and return false
			if (IsAttacked(WhereKing(0),7) > 0)
			{
				RemovePiece(toSquare,2);
				AddPiece(fromSquare,dep);
				AddPiece(toSquare,dest);
				return false;
			}
			
			EnPassant = 0;	
			ChangeMovingSide();	
		}
		//black's 
		if (fromSquare > 7 && fromSquare < 16)
		{
			RemovePiece(fromSquare,dep);
			RemovePiece(toSquare,dest);
			AddPiece(toSquare,9);

			//if after making the move king is on check
			//then the move is invalid, 
			//make it back and return false
			if (IsAttacked(WhereKing(7),0) > 0)
			{
				RemovePiece(toSquare,9);
				AddPiece(fromSquare,dep);
				AddPiece(toSquare,dest);
				return false;
			}
			
			EnPassant = 0;			
			ChangeMovingSide();
		}

		//if a rook, which can castle, was captured,
		//the corresponding possibility is lost.

		if ((dest == 3) && (toSquare == 0) && (WhiteLongCastling == true))
			WhiteLongCastling = false;

		if ((dest == 3) && (toSquare == 7) && (WhiteShortCastling == true))
			WhiteShortCastling = false;

		if ((dest == 10) && (toSquare == 56) && (BlackLongCastling == true))
			BlackLongCastling = false;

		if ((dest == 10) && (toSquare == 63) && (BlackShortCastling == true))
			BlackShortCastling = false;
		
		return true;
	
	}

	return false;
}


///////////////////////////////////////////
//this function calculates the first hash key of the current position
///////////////////////////////////////////
__int32 ComputePrimaryHashKey(void)
{
	__int32 key = 0;
    __int64 bbWhite=0, 
			bbBlack=0;

    for( int WhitePiece=1; WhitePiece<7; WhitePiece++)
	{
		bbWhite = BitBoards[WhitePiece];
     
		for( int square = 0; square < 64; square++ )
		{
			//Zobrist's method of assigning a random value
			//to a certain position
			if ( ( bbWhite & Squares[square] ) != 0 )
				key^=PrimaryHashKeys[WhitePiece-1][square];
		}
    }

    for( int BlackPiece=8; BlackPiece<14; BlackPiece++)
	{
		bbBlack = BitBoards[BlackPiece];
      
		for( int square = 0; square < 64; square++ )
		{
			//Zobrist's method of assigning a random value
			//to a certain position
			if ( ( bbBlack & Squares[square] ) != 0 )
				key^=PrimaryHashKeys[BlackPiece-2][square];
		}
    }

	//If EnPassant status is different,
	//the position is different
	if (EnPassant != 0)
		key ^= EPHashKey;

	//if Black is on move, 
	//the position index differs
	if (MovingSide == 7)
		key ^= BlackMoveHashKey;

return key;
}


///////////////////////////////////////////
//this function calculates the second hash key of the current position
///////////////////////////////////////////
__int32 ComputeSecondaryHashKey(void)
{
	__int32 key = 0;
    __int64 bbWhite, 
			bbBlack;

    for( int WhitePiece=1; WhitePiece<7; WhitePiece++)
	{
		bbWhite = BitBoards[WhitePiece];
      
		for( int square = 0; square < 64; square++ )
		{
			//Zobrist's method of assigning a random value
			//to a certain position
			if ( ( bbWhite & Squares[square] ) != 0 )
				key^=SecondaryHashKeys[WhitePiece-1][square];
		}
    }

    for( int BlackPiece=8; BlackPiece<14; BlackPiece++)
	{
		bbBlack = BitBoards[BlackPiece];
      
		for( int square = 0; square < 64; square++ )
		{
			//Zobrist's method of assigning a random value
			//to a certain position
			if ( ( bbBlack & Squares[square] ) != 0 )
				key^=SecondaryHashKeys[BlackPiece-2][square];
		}
    }

	//If EnPassant status is different,
	//the position is different
	if (EnPassant != 0)
		key ^= EPHashKey;

	//if Black is on move, 
	//the position index must differ
	if (MovingSide == 7)
		key ^= BlackMoveHashKey;

return key;
}


///////////////////////////////////////////
//this function computes all legal moves of a given piece 
//on a given square in the current position
///////////////////////////////////////////
__int64 ComputeMoves(int piece, int square)
{
	__int64 tmp=0;
	int j=0;

	//////////king's moves
	if (piece == 1 || piece == 8)
	{
		//king cannot capture friendly pieces
		if (piece == 1 )
			tmp = KingMovesBB[square] & (~BitBoards[0]);
		else 
			tmp = KingMovesBB[square] & (~BitBoards[7]);
	
		//castling moves
		
		//White's
		//king must stay on E1, must not be in check,
		//the side to move must still have a possibility to castle
		if(piece == 1 && square == 4 && IsAttacked(4,7) == 0)
		{
			//is short castling possible ?
			if(WhiteShortCastling == true)
			{
				//are F1(5) and G1(6) empty and not attacked by Blacks?
				if ((WhichPiece(5) == -1) && (IsAttacked(5,7) == 0) && 
					(WhichPiece(6) == -1) && (IsAttacked(6,7) == 0))
					tmp |= Squares[6];
			}
			//is long castling possible ?
			if(WhiteLongCastling == true)
			{
				//are D1(3) and C1(2) empty and not attacked by Blacks and B1(1) empty?
				if ( ( (WhichPiece(3) == -1) && (IsAttacked(3,7) == 0) ) && 
					 ( (WhichPiece(2) == -1) && (IsAttacked(2,7) == 0) ) &&
					   (WhichPiece(1) == -1))//&& (IsAttacked(1,7) == 0))
					tmp |= Squares[2];
			}
		}
	
		//Black's
		//king must stay on E8, must not be in check,
		//the side to move must still have a possibility to castle
		if(piece == 8 && square == 60 && IsAttacked(60,0) == 0)
		{
			//is short castling possible ?
			if(BlackShortCastling == true)
			{
				//are F8(61) and G8(62) empty and not attacked by Whites?
				if ((WhichPiece(61) == -1) && (IsAttacked(61,0) == 0) && 
					(WhichPiece(62) == -1) && (IsAttacked(62,0) == 0))
					tmp |= Squares[62];
			}
			//is long castling possible ?
			if(BlackLongCastling == true)
			{
				//are D8(59) and C8(58) empty and not attacked by Whites and B8(57) empty?
				if ( ( (WhichPiece(59) == -1) && (IsAttacked(59,0) == 0) ) && 
					 ( (WhichPiece(58) == -1) && (IsAttacked(58,0) == 0) ) &&
					   (WhichPiece(57) == -1)) //&& (IsAttacked(57,0) == 0))
					tmp |= Squares[58];
			}
		}
	}

	//////////knight's moves
	if (piece == 5 || piece == 12)
	{
		//knight cannot capture friendly pieces
		if (piece == 5)
			tmp = KnightMovesBB[square] & (~BitBoards[0]);
		else 
			tmp = KnightMovesBB[square] & (~BitBoards[7]);
	}

	//////////rook's moves
	if (piece == 3 || piece == 10)
	{
		for (int direction = 0; direction < 4; direction++)
		{
			j=0;
			while((RookMoves[square][direction][j] != -1) && j<7 )
			{
				//if the square is empty,
				//then it is surely a possible move 
				if (WhichPiece(RookMoves[square][direction][j]) == -1)
				{
					tmp |= Squares[RookMoves[square][direction][j]];
					j++;
				}
				//if the square is NOT empty
				else
				{
					//the square is occupied by a friendly piece
					if ((WhichPiece(RookMoves[square][direction][j]) < 7 && piece == 3) || 
						(WhichPiece(RookMoves[square][direction][j]) > 7 && piece == 10) )
						//it is not a possible move
						break;
					//by an enemie's one
					else 
					{
						//it is a possible (capture) move
						tmp |= Squares[RookMoves[square][direction][j]];
						break;
					}
				}
			}// end of while
		}// end of for
	}

	//////////bishop's moves 
	if (piece == 4 || piece == 11)
	{
		for (int direction = 0; direction < 4; direction++)
		{
			j=0;
			while((BishopMoves[square][direction][j] != -1) && j<7 )
			{
				//if the square is empty,
				//then it is surely a possible move 
				if (WhichPiece(BishopMoves[square][direction][j]) == -1)
				{
					tmp |= Squares[BishopMoves[square][direction][j]];
					j++;
				}
				//if the square is NOT empty
				else
				{
					//the square is occupied by a friendly piece
					if ((WhichPiece(BishopMoves[square][direction][j]) < 7 && piece == 4) || 
						(WhichPiece(BishopMoves[square][direction][j]) > 7 && piece == 11) )
						//it is not a possible move
						break;
					//by an enemie's one
					else 
					{
						//it is a possible (capture) move
						tmp |= Squares[BishopMoves[square][direction][j]];
						break;
					}
				}
			}// end of while
		}// end of for
	}

	//////////queen's moves
			//are at the same time the bishop's and the rook's ones
			//so two loops, already created above, are sequentially made
	if (piece == 2 || piece == 9)
	{
		//queen-as-rook
		for (int direction = 0; direction < 4; direction++)
		{
			j=0;
			while((RookMoves[square][direction][j] != -1) && j<7 )
			{
				//if the square is empty,
				//then it is surely a possible move 
				if (WhichPiece(RookMoves[square][direction][j]) == -1)
				{
					tmp |= Squares[RookMoves[square][direction][j]];
					j++;
				}
				//if the square is NOT empty
				else
				{
					//the square is occupied by a friendly piece
					if ((WhichPiece(RookMoves[square][direction][j]) < 7 && piece == 2) || 
						(WhichPiece(RookMoves[square][direction][j]) > 7 && piece == 9) )
						break;
					//by an enemie's one
					else 
					{
						tmp |= Squares[RookMoves[square][direction][j]];
						break;
					}
				}
			}// end of while
		}// end of queen-as-rook's for-loop

		//queen-as-bishop
		for (direction = 0; direction < 4; direction++)
		{
			j=0;
			while((BishopMoves[square][direction][j] != -1) && j<7 )
			{
				//if the square is empty,
				//then it is surely a possible move 
				if (WhichPiece(BishopMoves[square][direction][j]) == -1)
				{
					tmp |= Squares[BishopMoves[square][direction][j]];
					j++;
				}
				//if the square is NOT empty
				else
				{
					//the square is occupied by a friendly piece
					if ((WhichPiece(BishopMoves[square][direction][j]) < 7 && piece == 2) || 
						(WhichPiece(BishopMoves[square][direction][j]) > 7 && piece == 9) )
						break;
					//by an enemie's one
					else 
					{
						tmp |= Squares[BishopMoves[square][direction][j]];
						break;
					}
				}
			}// end of while
		}// end of queen-as-bishop's for loop
	}
	
	//////////white pawn's moves
	if (piece == 6)
	{
		//EnPassant capture moves
		if (EnPassant != 0)
		{
			if (EnPassant == 1)
			{
				if (square == 33)
					tmp |= Squares[square+7];
			}
			else 
			{
				if (EnPassant == 8)
				{	
					if (square == 38)
						tmp |= Squares[square+9];
				}
				//EnPassant = 2..7
				else
				{
					if ((square == EnPassant + 30) || (square == EnPassant + 32))
						tmp |= Squares[EnPassant + 39];
				}
			}		
		}
				
		//not EnPassant capture moves
		if (square ==  8 || square == 16 || square == 24 ||
			square == 32 || square == 40 || square == 48)
			{
				if (WhichPiece(square+9) > 8)
					tmp |= Squares[square+9];
			}
		else
			if (square == 15 || square == 23 || square == 31 ||
				square == 39 || square == 47 || square == 55)
			{
				if (WhichPiece(square+7) > 8)
					tmp |= Squares[square+7];
			}
			else 
			{
				if (square > 8 && square < 55)
				{
					if (WhichPiece(square+7) > 8)
						tmp |= Squares[square+7];
					if (WhichPiece(square+9) > 8)
						tmp |= Squares[square+9];
				}
			}

		//now ordinary moves
		if (WhichPiece(square+8) != -1)
			;
		else 
			{
				tmp|= Squares[square+8];
				//passing-two-squares move from the second line
				if ((square > 7 && square < 16) && WhichPiece(square+16) == -1)
					tmp |= Squares[square+16];
			}
	}//end of white-pawn-if

	//////////black pawn's moves
	if (piece == 13)
	{
		//EnPassant capture moves
		if (EnPassant != 0)
		{
			if (EnPassant == 1)
			{
				if (square == 25)
					tmp |= Squares[square-9];
			}
			else 
			{
				if (EnPassant == 8 )
				{	
					if (square == 30)
						tmp |= Squares[square-7];
				}
				//EnPassant = 2..7
				else
				{
					if ((square == EnPassant + 22) || (square == EnPassant + 24))
						tmp |= Squares[EnPassant + 15];
				}
			}		
		}

		//not EnPassant capture moves
		if (square == 55 || square == 47 || square == 39 ||
			square == 31 || square == 23 || square == 15)
			{
				if ((WhichPiece(square-9) > 1) && (WhichPiece(square-9) < 7))
					tmp |= Squares[square-9];
			}
		else
			if (square == 48 || square == 40 || square == 32 ||
				square == 24 || square == 16 || square == 8)
			{
				if ((WhichPiece(square-7) > 1) && (WhichPiece(square-7) < 7))
					tmp |= Squares[square-7];
			}
			else 
			{
				if (square < 55 && square > 8)
				{
					if ((WhichPiece(square-7) > 1) && (WhichPiece(square-7) < 7))
						tmp |= Squares[square-7];
					if ((WhichPiece(square-9) > 1) && (WhichPiece(square-9) < 7))
						tmp |= Squares[square-9];
				}
			}

		//all other moves
		if (WhichPiece(square-8) != -1)
			;
		else 
			{
				tmp|= Squares[square-8];
				//passing-two-squares move from the 7th line
				if ((square > 47 && square < 56) && WhichPiece(square-16) == -1)
					tmp |= Squares[square-16];
			}
	}//end of black-pawn-if

return tmp;
}


///////////////////////////////////////////
//this function loads evaluation parameters from a file.
//order is important (see thesis)!
///////////////////////////////////////////
///////////////////////////////////////////
bool LoadParameters( int *Chr)
{
	
	FILE *file;
	//there must be at largest 4-digited parameteres
	//where, hence, the 5th symbol is the symbol of string's end
	char param[5];
	int i=0,
		k=0;
	
	//NOTE! 
	//the full path must be specified!
	file=fopen("d:\\temp\\par.txt","r");

	if(!file)
	{
		printf("Open error\n");
		return false;
	}
	
	param[0]='\0';

	while(fgets(param,5,file))
	{
		k=0;
		while (param[k] >= '0' && param[k] <='9') 
		{
			Chr[i] = (Chr[i]*10) + (param[k] - '0');
			k++;
		}
		i++;
	}
	
	fclose(file);

	return true;
}


///////////////////////////////////////////
//this function checks if current position is valid
///////////////////////////////////////////
bool PositionValidity(void)
{
	int i=0,
		WhiteKingFlag=0,
		BlackKingFlag=0,
		WhitePawnFlag=0,
		BlackPawnFlag=0;

	//checking for kings and pawns on the board
	for(i=0;i<64;i++)
	{
		//white king
		if((Squares[i] & BitBoards[1]) != 0)
			WhiteKingFlag++;

		//black king
		if((Squares[i] & BitBoards[8]) != 0)
			BlackKingFlag++;

		//white pawns
		if((Squares[i] & BitBoards[6]) != 0)
			WhitePawnFlag++;

		//black pawns
		if((Squares[i] & BitBoards[13]) != 0)
			BlackPawnFlag++;

		//pawn positions
		if ((i<8) || (i>55))
		{
			if ( ((Squares[i] & BitBoards[6]) != 0) || ((Squares[i] & BitBoards[13]) != 0))
			{
				printf("Invlaid position: pawns on the first or last line\n");
				return false;
			}
		}

	}

	if (WhiteKingFlag == 0)
	{
		printf("Invalid position: No white king\n");
		return false;
	}
	else
		if (WhiteKingFlag > 1)
		{
			printf("Invalid position: Too many white kings\n");
			return false;
		}

	if (BlackKingFlag == 0)
	{
		printf("Invalid position: No black king\n");
		return false;
	}
	else
		if (BlackKingFlag > 1)
		{
			printf("Invalid position: Too many black kings\n");
			return false;
		}

	if (WhitePawnFlag > 8)
	{
		printf("Invalid position: Too many white pawns\n");
		return false;
	}

	if (BlackPawnFlag > 8)
	{
		printf("Invalid position: Too many black pawns\n");
		return false;
	}

	//checking "who starts" possibilities
	if (MovingSide == 0)
	{
		if (IsAttacked(WhereKing(7),0) > 0)
		{
			printf("White cannot move now: black king is on check\n");
			return false;
		}
	}
	else
	{
		if (IsAttacked(WhereKing(0),7) > 0)
		{
			printf("Black cannot move now: white king is on check\n");
			return false;
		}
	}

return true;
}


///////////////////////////////////////////
//this function transforms square number to its usual notation,
//i.e. 0-->a1, 63-->h8
///////////////////////////////////////////
bool NumberToField(int number, char *field)
{
	int q = 0,
		r = 0;
	
	field[0]='\0';
	
	if ((number < 0) || (number > 63))
	{
		printf("Illegal number\n");
		return false;
	}
	else
	{
		r = number % 8;

		if (r == 0)
			field[0] = 'a';
		else
			if (r == 1)
				field[0] = 'b';
			else
				if (r == 2)
					field[0] = 'c';
				else
					if (r == 3)
						field[0] = 'd';
					else
						if (r == 4)
							field[0] = 'e';
						else
							if (r == 5)
								field[0] = 'f';
							else
								if (r == 6)
									field[0] = 'g';
								else
									field[0] = 'h';
		q = (number - r)/8;

		if (q == 0)
			field[1] = '1';
		else
			if (q == 1)
				field[1] = '2';
			else
				if (q == 2)
					field[1] = '3';
				else
					if (q == 3)
						field[1] = '4';
					else
						if (q == 4)
							field[1] = '5';
						else
							if (q == 5)
								field[1] = '6';
							else
								if (q == 6)
									field[1] = '7';
								else
									field[1] = '8';
	}
		
	field[2] = '\0';
	return true;
}


///////////////////////////////////////////
//this function transforms usual notation to square number,
//i.e. a1-->0, h8-->63
///////////////////////////////////////////
int FieldToNumber(char *field)
{
	//char temp[2];
	int q = 0,
		r = 0;
	
	//field[0]='\0';
	
	if ((field[0] < 'a') || (field[0] > 'h'))
	{
		printf("Illegal field\n");
		return -1;
	}
	else
		if ((field[1] < '1') || (field[1] > '8'))
		{
			printf("Illegal field\n");
			return -1;
		}
		else
		{

		if (field[1] == '1')
			q = 0;
		else 
			if (field[1] == '2')
				q = 1;
			else 
				if (field[1] == '3')
					q = 2;
				else 
					if (field[1] == '4')
						q = 3;
					else 
						if (field[1] == '5')
							q = 4;
						else 
							if (field[1] == '6')
								q = 5;
							else 
								if (field[1] == '7')
									q = 6;
								else 
									q = 7;



		if (field[0] == 'a')
			r = 0;
		else
			if (field[0] == 'b')
				r = 1;
			else
				if (field[0] == 'c')
					r = 2;
				else
					if (field[0] == 'd')
						r = 3;
					else
						if (field[0] == 'e')
							r = 4;
					else
						if (field[0] == 'f')
							r = 5;
						else
							if (field[0] == 'g')
								r = 6;
							else 
								r = 7;
	}
		
	return (8*q + r);
}

///////////////////////////////////////////
//this function provides the very basic interface with 
//Tim Mann's WinBoard (considered and tested) or/and xboard (not considered, not tested)
//See more at http://www.tim-mann.org/xboard.html
//Also http://home.comcast.net/~tckerrigan/ should be useful
///////////////////////////////////////////
void winboard()
{
		//different auxilary variables
	int Computer = 6,
		Sc = 0,
		HashValues[100][3],
		CurrentIndex = 0,
		CurrentMoveType,
		CurrentMovingPiece,
		ResignFlag = 0;
	
    char line[256], 
		 command[256],
		 fromSquare[3],
		 toSquare[3],
		 //ideally, this variable must be of length 5
		 //since there is a promotion move s1s2X.
		 //but as only promotion toQueen is implemented,
		 //Q is added manually each time
		 //Moreover, the situation of non-queen promotion
		 //usually must be handled in a different way specifically
		 usermove[4];

   	//for 50 moves rule
	for (int z = 0; z < 50; z++)
	{
		HashValues[z][0]=-1;
		HashValues[z][1]=-1;
		HashValues[z][2]=1;
	}

	//pre-start cleaning
	usermove[0]='\0';
	fromSquare[0]='\0';
	toSquare[0]='\0';

	//required by application
	signal(SIGINT,SIG_IGN);
	printf("\n");

	//initialise the game
	Initialize();
	InitialPosition();
	
	//the game (see more in the thesis about the style)
	for (;;) 
	{
		fflush(stdout);

		//computer's turn to make the move
		if(MovingSide == Computer)
	    {
			fromSquare[0]='\0';
			toSquare[0]='\0';

			//find the move
			Sc = FindMove(3);

			//own king checkmated (i.e. no move)
			if (Sc == -20000)
			{
				printf("checkmate\n");
				Computer = 6;
			}
			//opponent's king checkmated
			else 
				if (Sc == 20000)
					{
						NumberToField(BestFromSquare,fromSquare);
						NumberToField(BestToSquare,toSquare);
			
						CurrentMoveType  = SetMoveType(BestFromSquare,BestToSquare);
						CurrentMovingPiece = WhichPiece(BestFromSquare);

						if (Move(BestFromSquare,BestToSquare,SetMoveType(BestFromSquare,BestToSquare)) == true)
						{
							if ((CurrentMoveType == 401) || (CurrentMoveType == 411))
							{
								printf("move %s%sQ\n",fromSquare,toSquare);
								printf("checkmate\n");
							}
							else
							{
								printf("move %s%s\n",fromSquare,toSquare);
								printf("checkmate\n");
							}
					
						}
					}
				//bad only move found
				else
				if (Sc <= -500)
				{
					ResignFlag++;
					
					//if for the secod time in a row, then resign
					if (ResignFlag == 2)
					{
						printf("resign\n");
						Computer = 6;
					}
					//first bad move
					else
					{
						NumberToField(BestFromSquare,fromSquare);
						NumberToField(BestToSquare,toSquare);
			
						CurrentMoveType  = SetMoveType(BestFromSquare,BestToSquare);
						CurrentMovingPiece = WhichPiece(BestFromSquare);

						if (Move(BestFromSquare,BestToSquare,SetMoveType(BestFromSquare,BestToSquare)) == true)
						{
							if ((CurrentMoveType == 401) || (CurrentMoveType == 411))
								printf("move %s%sQ\n",fromSquare,toSquare);
							else
								printf("move %s%s\n",fromSquare,toSquare);
					
							if (CheckPosition(CurrentMoveType, CurrentMovingPiece, HashValues, CurrentIndex) == true)
							{
								printf("offer draw\n");
								Computer = 6;
							}
						}
					}
				}
				//if there is an acceptable move
				else
				{
					NumberToField(BestFromSquare,fromSquare);
					NumberToField(BestToSquare,toSquare);
			
					CurrentMoveType  = SetMoveType(BestFromSquare,BestToSquare);
					CurrentMovingPiece = WhichPiece(BestFromSquare);

					if (Move(BestFromSquare,BestToSquare,SetMoveType(BestFromSquare,BestToSquare)) == true)
					{
						ResignFlag = 0;
						if ((CurrentMoveType == 401) || (CurrentMoveType == 411))
							printf("move %s%sQ\n",fromSquare,toSquare);
						else
							printf("move %s%s\n",fromSquare,toSquare);
					
						if (CheckPosition(CurrentMoveType, CurrentMovingPiece, HashValues, CurrentIndex) == true)
						{
							printf("offer draw\n");
							Computer = 6;
						}
					}
				}

            continue;
        }//end of if(MovingSide == Computer)

		//get a string from a stream
		//and save it in 'line'
		if (!fgets(line, 256, stdin))
			return;

		if (line[0] == '\n')
			continue;

		//remember the input
		sscanf(line, "%s", command);

        ///////////////////
		//handling protocol
		///////////////////
		if (!strcmp(command, "protover"))
	    {
			printf("feature time=0 colors=0 done=1\n");
			continue;
        }

        if (!strcmp(command, "draw")) 
		{
			printf("offer draw\n");
			Computer = 6;
			continue;
		}
        
        if (!strcmp(command, "result")) 
		{
			Computer=6;
			continue;
		}

		if (!strcmp(command, "xboard"))
			continue;

		if (!strcmp(command, "new")) 
		{
			Initialize();
			InitialPosition();
		  	for (int z = 0;z < 50; z++)
			{
				HashValues[z][0]=-1;
				HashValues[z][1]=-1;
				HashValues[z][2]=1;
			}
			CurrentIndex = 0;

			Computer = 7;
			continue;
		}

		if (!strcmp(command, "random"))
			continue;
		
		if (!strcmp(command, "quit"))
			return;
		
		if (!strcmp(command, "force")) 
		{
			Computer = 6;
			continue;
		}
		
		if (!strcmp(command, "computer")) 
			continue;

		if (!strcmp(command, "go")) 
		{
			Computer = MovingSide;
			continue;
		}
		
		if (!strcmp(command, "level"))
			continue;
       
		if (!strcmp(command, "post")) 
			continue;
		
		if (!strcmp(command, "hard")) 
			continue;
		
		if (!strcmp(command, "accepted")) 
			continue;
        
		if (!strcmp(command, "rejected")) 
			continue;

		//parsing different from all above user's input
		strcpy(usermove,line);
		
		fromSquare[0] = usermove[0];
		fromSquare[1] = usermove[1];
		fromSquare[2] = '\0';

		toSquare[0] = usermove[2];
		toSquare[1] = usermove[3];
		toSquare[2] = '\0';

 		CurrentMoveType  = SetMoveType(FieldToNumber(fromSquare),FieldToNumber(toSquare));
		CurrentMovingPiece = WhichPiece(FieldToNumber(fromSquare));
       
		//making opponent's move, if any
		if ((IsMoveValid(FieldToNumber(fromSquare),FieldToNumber(toSquare)) == false) || 
			(Move(FieldToNumber(fromSquare),FieldToNumber(toSquare),SetMoveType(FieldToNumber(fromSquare),FieldToNumber(toSquare))) == false))
		{
			printf("move illegal\n");
		}
		else
		{
			if (CheckPosition(CurrentMoveType, CurrentMovingPiece, HashValues, CurrentIndex) == true)
			{
				printf("offer draw\n");
				Computer = 6;
			}
		}
	}

}


///////////////////////////////////////////
//this function checks for game break options,
// i.e. 3-time repetition, 50 moves draw
///////////////////////////////////////////
bool CheckPosition(int CurrentMoveType, int CurrentMovingPiece, int HashValues[100][3], int& CurrentIndex)
{
	int res = 0;
	int CurrentPrimaryHashValue = ComputePrimaryHashKey();
	int CurrentSecondaryHashValue = ComputeSecondaryHashKey();				

	//3 times repetition draw
	for (int k = (CurrentIndex-1); k >= 0; k--)
	{
		if (CurrentPrimaryHashValue == HashValues[k][0])
			if (CurrentSecondaryHashValue == HashValues[k][1])
				HashValues[CurrentIndex][2] = HashValues[k][2] + 1;

		HashValues[CurrentIndex][0] = CurrentPrimaryHashValue;
		HashValues[CurrentIndex][1] = CurrentSecondaryHashValue;

		if (HashValues[CurrentIndex][2] == 3)
		{
			return true;
			break;
		}
	}

	CurrentIndex++;
	CurrentIndex = CurrentIndex%50;

	//50 moves draw situation
	if (CurrentMoveType == 2)
		_50MovesDraw++;
	else
		if (CurrentMoveType == 0)
		{
			if ((CurrentMovingPiece != 6) && (CurrentMovingPiece != 13))
				_50MovesDraw++;
			else 
				_50MovesDraw = 0;
		}
		else 
			_50MovesDraw = 0;
					
	if (_50MovesDraw >= 100)
		return true;
	
	return false;
}


///////////////////////////////////////////
//this function checks if a move is inside the chess board
///////////////////////////////////////////
bool IsMoveValid(int fromSquare, int toSquare)
{
	if ((fromSquare < 0) || (fromSquare > 63) || (toSquare < 0) || (toSquare > 63))
		return false;

	return true;
}