///////////////////////////////////////////
//this function changes the moving side in the current position
///////////////////////////////////////////
void ChangeMovingSide(void);


///////////////////////////////////////////
//this function prepares everything for the start
///////////////////////////////////////////
bool Initialize(void);


///////////////////////////////////////////
//this function prints the bit board in the following manner:
//
//a1...h1
//.......
//a8...h8
//That's why Inverse...
///////////////////////////////////////////
void InverseBitRepresentation( __int64 x);


///////////////////////////////////////////
//this function returns the number of a piece on the square
//or -1 if the square is empty.
///////////////////////////////////////////
int WhichPiece(int square);


///////////////////////////////////////////
//this function returns the number of square, on which the king of 
//the desired side stays.
//Assumes that the king is present!!!
///////////////////////////////////////////
int WhereKing(int side);


///////////////////////////////////////////
//this function shows the current chessboard in "numberous" style, 
//i.e. white king = 1, white queen = 2, empty square = -1
///////////////////////////////////////////
void ShowBoard(void);


///////////////////////////////////////////
//this function returns the number of attacks 
//on the desired square by the desired side
///////////////////////////////////////////
int IsAttacked(int square, int ByWhichSide);


///////////////////////////////////////////
//this function adds the desired piece onto the desired square
///////////////////////////////////////////
void AddPiece(int square, int piece);


///////////////////////////////////////////
//this function removes certain piece from the mentioned square
///////////////////////////////////////////
void RemovePiece(int square, int piece);


///////////////////////////////////////////
//this function creates the initial chess board position
///////////////////////////////////////////
void InitialPosition();


///////////////////////////////////////////
//this function determines the type of the move
///////////////////////////////////////////
int SetMoveType(int fromSquare, int toSquare);


///////////////////////////////////////////
//This function makes the move,
//Type must match to the real type of fromSquare-toSquare move.
//So using Type = SetMoveType(fromSquare,toSquare) prevents the possible error of MoveType mismatch
///////////////////////////////////////////
bool Move(int fromSquare, int toSquare, int Type);


///////////////////////////////////////////
//this function calculates the first hash key of the current position
///////////////////////////////////////////
__int32 ComputePrimaryHashKey(void);


///////////////////////////////////////////
//this function calculates the second hash key of the current position
///////////////////////////////////////////
__int32 ComputeSecondaryHashKey(void);


///////////////////////////////////////////
//this function computes all legal moves of a given piece 
//on a given square in the current position
///////////////////////////////////////////
__int64 ComputeMoves(int piece, int square);


///////////////////////////////////////////
//this function loads evaluation parameters from a file.
//order is important (see thesis)!
///////////////////////////////////////////
bool LoadParameters( int *Chr);


///////////////////////////////////////////
//this function checks if current position is valid
///////////////////////////////////////////
bool PositionValidity(void);


///////////////////////////////////////////
//this function transforms square number to its usual notation,
//i.e. 0-->a1, 63-->h8
///////////////////////////////////////////
bool NumberToField(int number, char *field);


///////////////////////////////////////////
//this function transforms usual notation to square number,
//i.e. a1-->0, h8-->63
///////////////////////////////////////////
int FieldToNumber(char *field);


///////////////////////////////////////////
//this function provides the very basic interface with 
//Tim Mann's WinBoard (tested) or/and xboard (not tested)
//See more at http://www.tim-mann.org/xboard.html
//Also http://home.comcast.net/~tckerrigan/  should be useful
///////////////////////////////////////////
void winboard(void);


///////////////////////////////////////////
//this function checks for game break options,
// i.e. 3-time repetition, 50 moves draw
///////////////////////////////////////////
bool CheckPosition(int CurrentMoveType, int CurrentMovingPiece, int HashValues[100][3], int& CurrentIndex);


///////////////////////////////////////////
//this function checks if a move is inside the chess board
///////////////////////////////////////////
bool IsMoveValid(int fromSquare, int toSquare);